import { QueryBuilderState } from '../store/querybuilderSlice';
import { URLParams, setParam } from '../api/url';
import { Broker } from './broker';
import { DateStringStatement } from '../../querybuilder/model/logic/general';

export const databaseNameMapping: string[] = ['arangodb', 'neo4j'];
export const databaseProtocolMapping: string[] = ['neo4j://', 'neo4j+s://', 'bolt://', 'bolt+s://'];

export enum DatabaseType {
  ArangoDB = 0,
  Neo4j = 1,
}

export enum DatabaseStatus {
  untested = 0,
  testing = 1,
  tested = 2,
}

export type DatabaseInfo = {
  internalDatabaseName: string;
  url: string;
  port: number;
  protocol: string;
  username: string;
  password: string;
  type: number;
};

export const nilUUID = '00000000-0000-0000-0000-000000000000';

export type SaveStateI = {
  id: string;
  user_id: string;
  name: string;
  db: DatabaseInfo;
  schema: any;
  queryBuilder: QueryBuilderState;
  visualization: any;
  share_state: any;
};

type GetStateResponse = (data: SaveStateI) => void;
export function wsGetState(saveStateId: string, callback?: GetStateResponse) {
  Broker.instance().sendMessage(
    {
      key: 'state',
      subKey: 'get',
      body: { saveStateId: saveStateId }, //messageTypeGetSaveState
    },
    callback,
  );
}
export function wsGetStateSubscription(callback: GetStateResponse) {
  const id = Broker.instance().subscribe(callback, 'save_state');
  return () => {
    Broker.instance().unSubscribe('save_state', id);
  };
}

type GetStatesResponse = (data: SaveStateI[]) => void | boolean;
export function wsGetStates(callback?: GetStatesResponse) {
  Broker.instance().sendMessage(
    {
      key: 'state',
      subKey: 'getAll',
    },
    callback,
  );
}
export function wsGetStatesSubscription(callback: GetStatesResponse) {
  const id = Broker.instance().subscribe(callback, 'save_states');
  return () => {
    Broker.instance().unSubscribe('save_states', id);
  };
}

type SelectStateResponse = (data: { saveStateID: string; success: boolean }) => void;
export function wsSelectState(saveStateId: string | undefined, callback?: SelectStateResponse) {
  if (saveStateId === undefined) saveStateId = '';
  Broker.instance().sendMessage(
    {
      key: 'state',
      subKey: 'select',
      body: { saveStateId: saveStateId }, //messageTypeGetSaveState
    },
    callback,
  );

  Broker.instance().useSaveStateID(saveStateId);
  setParam(URLParams.saveState, saveStateId);
}
export function wsSelectStateSubscription(callback: SelectStateResponse) {
  const id = Broker.instance().subscribe(callback, 'save_state_selected');
  return () => {
    Broker.instance().unSubscribe('save_state_selected', id);
  };
}

export function wsCreateState(request: SaveStateI, callback?: GetStateResponse) {
  Broker.instance().sendMessage(
    {
      key: 'state',
      subKey: 'create',
      body: request, //SaveStateModel
    },
    callback,
  );
  // Also returns save_state
}

type DeleteStateResponse = (status: 'deleted' | DateStringStatement) => void;
export function wsDeleteState(id: string, callback?: Function) {
  Broker.instance().sendMessage(
    {
      key: 'state',
      subKey: 'delete',
      body: { saveStateId: id }, //messageTypeGetSaveState
    },
    callback,
  );
}
export function wsDeleteStateSubscription(callback: DeleteStateResponse) {
  const id = Broker.instance().subscribe(callback, 'delete_save_state');
  return () => {
    Broker.instance().unSubscribe('delete_save_state', id);
  };
}

type TestSaveStateConnectionResponse = (data: { status: 'success' | 'fail'; saveStateID: string }) => void;
export function wsTestSaveStateConnection(id: string, callback?: Function) {
  Broker.instance().sendMessage(
    {
      key: 'state',
      subKey: 'testConnection',
      body: { saveStateId: id }, //messageTypeGetSaveState
    },
    callback,
  );
}
export function wsTestSaveStateConnectionSubscription(callback: TestSaveStateConnectionResponse) {
  const id = Broker.instance().subscribe(callback, 'tested_connection');
  return () => {
    Broker.instance().unSubscribe('tested_connection', id);
  };
}

export function wsUpdateState(request: SaveStateI, callback?: GetStateResponse) {
  console.log('wsUpdateState', request);
  Broker.instance().sendMessage(
    {
      key: 'state',
      subKey: 'update',
      body: request, //SaveStateModel
    },
    callback,
  );
  // Also returns save_state
}

export function wsTestDatabaseConnection(dbConnection: DatabaseInfo, callback?: TestSaveStateConnectionResponse) {
  Broker.instance().sendMessage(
    {
      key: 'dbConnection',
      subKey: 'testConnection',
      body: dbConnection, //DBConnectionModel
    },
    callback,
  );
}
