import React, { useEffect, useRef, useState } from 'react';
import { GraphType, LinkType, NodeType } from './types';
import { NLPixi } from './components/NLPixi';
import { parseQueryResult } from './components/query2NL';
import { useImmer } from 'use-immer';
import { ML, setShortestPathSource, setShortestPathTarget } from '../../../data-access/store/mlSlice';
import { Layouts } from '../../../graph-layout/types';
import { Input } from '@graphpolaris/shared/lib/components/inputs';
import { GraphMetadata } from '@graphpolaris/shared/lib/data-access/statistics';
import { SettingsContainer, SettingsHeader } from '@graphpolaris/shared/lib/vis/components/config';
import { VISComponentType, VisualizationPropTypes } from '../../common';
import { EntityPill } from '@graphpolaris/shared/lib/components/pills/Pill';
import { nodeColorHex } from './components/utils';

export interface NodelinkVisProps {
  layout: string;
  showPopUpOnHover: boolean;
  shapes: {
    similar: boolean;
    shape: 'circle' | 'rectangle';
    shapeMap: { [id: string]: 'circle' | 'rectangle' } | undefined;
  };
  edges: {
    width: {
      similar: boolean;
      width: number;
    };
  };
  nodeList: string[];
}

const configuration: NodelinkVisProps = {
  layout: Layouts.FORCEATLAS2WEBWORKER as string,
  showPopUpOnHover: true,
  shapes: {
    similar: true,
    shape: 'circle',
    shapeMap: undefined,
  },
  edges: {
    width: { similar: true, width: 0.2 },
  },
  nodeList: [],
};

export const NodeLinkVis = React.memo(({ data, ml, dispatch, configuration }: VisualizationPropTypes) => {
  const ref = useRef<HTMLDivElement>(null);
  const [graph, setGraph] = useImmer<GraphType | undefined>(undefined);
  const [highlightNodes, setHighlightNodes] = useState<NodeType[]>([]);
  const [highlightedLinks, setHighlightedLinks] = useState<LinkType[]>([]);
  //console.log('data ', data.metaData.nodes.labels);
  useEffect(() => {
    if (data) {
      setGraph(
        parseQueryResult(data, ml, {
          defaultX: (ref.current?.clientWidth || 1000) / 2,
          defaultY: (ref.current?.clientHeight || 1000) / 2,
        }),
      );
    }
  }, [data, ml]);

  const onClickedNode = (node: NodeType, ml: ML) => {
    if (graph) {
      if (ml.shortestPath.enabled) {
        setGraph((draft) => {
          let _node = draft?.nodes.find((n) => n.id === node.id);
          if (!_node) return draft;

          if (!ml.shortestPath.srcNode) {
            _node.isShortestPathSource = true;
            dispatch(setShortestPathSource(node.id));
          } else if (ml.shortestPath.srcNode === node.id) {
            _node.isShortestPathSource = false;
            dispatch(setShortestPathSource(undefined));
          } else if (!ml.shortestPath.trtNode) {
            _node.isShortestPathTarget = true;
            dispatch(setShortestPathTarget(node.id));
          } else if (ml.shortestPath.trtNode === node.id) {
            _node.isShortestPathTarget = false;
            dispatch(setShortestPathTarget(undefined));
          } else {
            _node.isShortestPathSource = true;
            _node.isShortestPathTarget = false;
            dispatch(setShortestPathSource(node.id));
            dispatch(setShortestPathTarget(undefined));
          }
          return draft;
        });
      }
    }
  };

  return (
    <NLPixi
      graph={graph}
      configuration={configuration}
      highlightNodes={highlightNodes}
      highlightedLinks={highlightedLinks}
      onClick={(node, pos) => {
        onClickedNode(node, ml);
      }}
      layoutAlgorithm={configuration.layout}
    />
  );
});

const NodelinkSettings = ({
  configuration,
  graphMetadata,
  updateSettings,
}: {
  configuration: NodelinkVisProps;
  graphMetadata: GraphMetadata;
  updateSettings: (val: any) => void;
}) => {
  useEffect(() => {
    if (graphMetadata && graphMetadata.nodes && graphMetadata.nodes.labels.length > 0) {
      updateSettings({ nodeList: graphMetadata.nodes.labels });
    }
  }, [graphMetadata]);

  return (
    <SettingsContainer>
      <div className="mb-4">
        <h1 className="text-sm font-bold">General</h1>
        <div className="m-1 flex flex-col space-y-4">
          <h1>Nodes Labels:</h1>
          {configuration.nodeList.map((item, index) => (
            <div className="flex m-1 items-center">
              <div className="w-3/4 mr-6">
                <EntityPill title={item} />
              </div>
              <div className="w-1/2">
                <div className={`h-5 w-5 border-2 border-sec-300`} style={{ backgroundColor: nodeColorHex(index + 1) }}></div>
              </div>
            </div>
          ))}
        </div>
        <Input
          type="dropdown"
          label="Layout"
          value={configuration.layout}
          options={Object.values(Layouts) as string[]}
          onChange={(val) => updateSettings({ layout: val })}
        />
        <Input
          type="boolean"
          label="Show pop-up on hover"
          value={configuration.showPopUpOnHover}
          onChange={(val) => updateSettings({ showPopUpOnHover: val })}
        />
      </div>

      <div className="mb-4">
        <h1 className="text-sm font-bold">Nodes</h1>

        <div>
          <span className="text-xs font-semibold">Shape</span>
          <Input
            type="boolean"
            label="Common shape?"
            value={configuration.shapes.similar}
            onChange={(val) => updateSettings({ shapes: { ...configuration.shapes, similar: val } })}
          />
          {configuration.shapes.similar ? (
            <Input
              type="dropdown"
              label="Shape"
              value={configuration.shapes.shape}
              options={['Circle', 'Square']}
              onChange={(val) => updateSettings({ shapes: { ...configuration.shapes, shape: val } })}
            />
          ) : (
            <span>Map shapes to labels (to be implemented)</span>
          )}
        </div>

        <div>
          <span className="text-xs font-semibold">Color</span>
        </div>
      </div>

      <div>
        <h1 className="text-sm font-bold">Edges</h1>
        <div>
          <span className="text-xs font-semibold">Edge width</span>
          <Input
            type="boolean"
            label="Common width"
            value={configuration.edges.width.similar}
            onChange={(val) => updateSettings({ edges: { ...configuration.edges, width: { ...configuration.edges.width, similar: val } } })}
          />
          <Input
            type="slider"
            label="Width"
            value={configuration.edges.width.width}
            onChange={(val) => updateSettings({ edges: { ...configuration.edges, width: { ...configuration.edges.width, width: val } } })}
            min={0.1}
            max={2}
            step={0.1}
          />
        </div>
      </div>
    </SettingsContainer>
  );
};

export const NodeLinkComponent: VISComponentType = {
  displayName: 'NodeLinkVis',
  component: NodeLinkVis,
  settings: NodelinkSettings,
  configuration: configuration,
};

export default NodeLinkComponent;
